from inline_snapshot import snapshot

from mirascope.llm import (
    AssistantMessage,
    Text,
    Thought,
    UserMessage,
)

sync_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "model_id": "anthropic-beta/claude-sonnet-4-0",
            "provider_model_name": "claude-sonnet-4-0",
            "params": {"thinking": False},
            "finish_reason": None,
            "usage": {
                "input_tokens": 68,
                "output_tokens": 8,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "raw": "BetaUsage(cache_creation=BetaCacheCreation(ephemeral_1h_input_tokens=0, ephemeral_5m_input_tokens=0), cache_creation_input_tokens=0, cache_read_input_tokens=0, input_tokens=68, output_tokens=8, server_tool_use=None, service_tier='standard')",
                "total_tokens": 76,
            },
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="How many primes below 400 contain 79 as a substring? Answer ONLY with the number, not sharing which primes they are."
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Thought(
                            thought="""\
I need to find all primes below 400 that contain "79" as a substring.

First, let me think about what numbers below 400 could contain "79" as a substring:
- Numbers of the form 79X (where X is a digit): 790, 791, 792, 793, 794, 795, 796, 797, 798, 799
- Numbers of the form X79 (where X is a digit): 179, 279, 379
- The number 79 itself

Wait, I need to be more systematic. Let me list all numbers below 400 that contain "79":

Two-digit: 79
Three-digit: 179, 279, 379, 790, 791, 792, 793, 794, 795, 796, 797, 798, 799

Wait, 790+ are all ≥ 400, so I don't need those.

So the candidates are: 79, 179, 279, 379

Now I need to check which of these are prime:

79: Let me check if 79 is prime.
Factors to check: 2, 3, 5, 7 (since √79 ≈ 8.9)
79 ÷ 2 = 39.5 (not divisible)
79 ÷ 3 = 26.33... (not divisible)
79 ÷ 5 = 15.8 (not divisible)
79 ÷ 7 = 11.28... (not divisible)
So 79 is prime.

179: Let me check if 179 is prime.
Factors to check: 2, 3, 5, 7, 11, 13 (since √179 ≈ 13.4)
179 ÷ 2 = 89.5 (not divisible)


179 ÷ 3 = 59.67... (not divisible)
179 ÷ 5 = 35.8 (not divisible)
179 ÷ 7 = 25.57... (not divisible)
179 ÷ 11 = 16.27... (not divisible)
179 ÷ 13 = 13.77... (not divisible)
So 179 is also prime.

I'll continue checking the remaining candidates systematically, testing divisibility by potential factors up to the square root of each number.\
"""
                        ),
                        Text(text="3"),
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "signature": "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",
                                "thinking": """\
I need to find all primes below 400 that contain "79" as a substring.

First, let me think about what numbers below 400 could contain "79" as a substring:
- Numbers of the form 79X (where X is a digit): 790, 791, 792, 793, 794, 795, 796, 797, 798, 799
- Numbers of the form X79 (where X is a digit): 179, 279, 379
- The number 79 itself

Wait, I need to be more systematic. Let me list all numbers below 400 that contain "79":

Two-digit: 79
Three-digit: 179, 279, 379, 790, 791, 792, 793, 794, 795, 796, 797, 798, 799

Wait, 790+ are all ≥ 400, so I don't need those.

So the candidates are: 79, 179, 279, 379

Now I need to check which of these are prime:

79: Let me check if 79 is prime.
Factors to check: 2, 3, 5, 7 (since √79 ≈ 8.9)
79 ÷ 2 = 39.5 (not divisible)
79 ÷ 3 = 26.33... (not divisible)
79 ÷ 5 = 15.8 (not divisible)
79 ÷ 7 = 11.28... (not divisible)
So 79 is prime.

179: Let me check if 179 is prime.
Factors to check: 2, 3, 5, 7, 11, 13 (since √179 ≈ 13.4)
179 ÷ 2 = 89.5 (not divisible)


179 ÷ 3 = 59.67... (not divisible)
179 ÷ 5 = 35.8 (not divisible)
179 ÷ 7 = 25.57... (not divisible)
179 ÷ 11 = 16.27... (not divisible)
179 ÷ 13 = 13.77... (not divisible)
So 179 is also prime.

I'll continue checking the remaining candidates systematically, testing divisibility by potential factors up to the square root of each number.\
""",
                                "type": "thinking",
                            },
                            {
                                "text": "3",
                                "type": "text",
                            },
                        ],
                    },
                ),
                UserMessage(
                    content=[
                        Text(
                            text="If you remember what the primes were, then share them, or say 'I don't remember.'"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[Text(text="I don't remember.")],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "text": "I don't remember.",
                                "type": "text",
                            }
                        ],
                    },
                ),
            ],
            "format": None,
            "tools": [],
        }
    }
)
async_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "model_id": "anthropic-beta/claude-sonnet-4-0",
            "provider_model_name": "claude-sonnet-4-0",
            "params": {"thinking": False},
            "finish_reason": None,
            "usage": {
                "input_tokens": 154,
                "output_tokens": 17,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "raw": "BetaUsage(cache_creation=BetaCacheCreation(ephemeral_1h_input_tokens=0, ephemeral_5m_input_tokens=0), cache_creation_input_tokens=0, cache_read_input_tokens=0, input_tokens=154, output_tokens=17, server_tool_use=None, service_tier='standard')",
                "total_tokens": 171,
            },
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="How many primes below 400 contain 79 as a substring? Answer ONLY with the number, not sharing which primes they are."
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Thought(
                            thought="""\
I need to find all prime numbers below 400 that contain "79" as a substring.

Let me think about this systematically. Numbers containing "79" as a substring could be:
- 79 itself
- Numbers of the form 79X (790, 791, 792, 793, 794, 795, 796, 797, 798, 799)
- Numbers of the form X79 (179, 279, 379)
- Numbers of the form X79Y (but these would be at least 1790, which is above 400)

Wait, let me be more careful. Numbers below 400 that contain "79" as a substring:

1) 79 itself
2) Numbers starting with 79: 790, 791, 792, 793, 794, 795, 796, 797, 798, 799 (but these are all ≥ 790, which is above 400)
3) Numbers ending with 79: 179, 279, 379
4) Numbers with 79 in the middle: these would need to be at least 3-digit numbers of the form A79B, so at least 1790, which is above 400.

So the candidates are: 79, 179, 279, 379.

Now let me check which of these are prime:

79: Let me check if 79 is prime.
- Not divisible by 2 (it's odd)
- Not divisible by 3: 7+9 = 16, not divisible by 3
- Not divisible by 5 (doesn't end in 0 or 5)
- Not divisible by 7: 79 ÷ 7 = 11.28..., so not divisible
- We only need to check up to √79 ≈ 8.9, so we need to check 2, 3, 5, 7, and 8 is not prime.


- 79 is prime.

I confirm 79 is indeed a prime number after carefully checking its divisibility.

Continuing the primality check for 179, I verify it's not divisible by key small primes. Checking up to its square root confirms 179 is prime.

For 279, I quickly determine it's not prime since it's divisible by 3, with 279 equaling 3 × 93.

Examining 379, I begin investigating its potential divisors, noting it's an odd number not immediately divisible by 2 or 3.

Systematically testing divisors up to its square root, I find no factors. Each division yields a non-integer result: 7, 11, 13, 17, and 19 do not divide 379 evenly.

Reaching √379, I conclude 379 is indeed prime.

Identifying the pattern, I recognize three primes containing "79": 79, 179, and 379.

Carefully examining potential two-digit and three-digit configurations below 400, I confirm these are the only valid primes matching the substring criteria.

Specifically, the numbers 79, 179, and 379 qualify as prime, while 279 is divisible by 3. By systematically checking each candidate number's divisibility and range constraints, I've determined there are precisely 3 primes containing "79" under 400.\
"""
                        ),
                        Text(
                            text="""\
Looking for primes below 400 that contain "79" as a substring.

The candidates are:
- 79
- 179  \n\
- 279
- 379

Checking primality:
- 79: prime
- 179: prime
- 279: divisible by 3 (2+7+9=18), so not prime
- 379: prime

3\
"""
                        ),
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "signature": " ",
                                "thinking": """\
I need to find all prime numbers below 400 that contain "79" as a substring.

Let me think about this systematically. Numbers containing "79" as a substring could be:
- 79 itself
- Numbers of the form 79X (790, 791, 792, 793, 794, 795, 796, 797, 798, 799)
- Numbers of the form X79 (179, 279, 379)
- Numbers of the form X79Y (but these would be at least 1790, which is above 400)

Wait, let me be more careful. Numbers below 400 that contain "79" as a substring:

1) 79 itself
2) Numbers starting with 79: 790, 791, 792, 793, 794, 795, 796, 797, 798, 799 (but these are all ≥ 790, which is above 400)
3) Numbers ending with 79: 179, 279, 379
4) Numbers with 79 in the middle: these would need to be at least 3-digit numbers of the form A79B, so at least 1790, which is above 400.

So the candidates are: 79, 179, 279, 379.

Now let me check which of these are prime:

79: Let me check if 79 is prime.
- Not divisible by 2 (it's odd)
- Not divisible by 3: 7+9 = 16, not divisible by 3
- Not divisible by 5 (doesn't end in 0 or 5)
- Not divisible by 7: 79 ÷ 7 = 11.28..., so not divisible
- We only need to check up to √79 ≈ 8.9, so we need to check 2, 3, 5, 7, and 8 is not prime.


- 79 is prime.

I confirm 79 is indeed a prime number after carefully checking its divisibility.

Continuing the primality check for 179, I verify it's not divisible by key small primes. Checking up to its square root confirms 179 is prime.

For 279, I quickly determine it's not prime since it's divisible by 3, with 279 equaling 3 × 93.

Examining 379, I begin investigating its potential divisors, noting it's an odd number not immediately divisible by 2 or 3.

Systematically testing divisors up to its square root, I find no factors. Each division yields a non-integer result: 7, 11, 13, 17, and 19 do not divide 379 evenly.

Reaching √379, I conclude 379 is indeed prime.

Identifying the pattern, I recognize three primes containing "79": 79, 179, and 379.

Carefully examining potential two-digit and three-digit configurations below 400, I confirm these are the only valid primes matching the substring criteria.

Specifically, the numbers 79, 179, and 379 qualify as prime, while 279 is divisible by 3. By systematically checking each candidate number's divisibility and range constraints, I've determined there are precisely 3 primes containing "79" under 400.\
""",
                                "type": "thinking",
                            },
                            {
                                "text": """\
Looking for primes below 400 that contain "79" as a substring.

The candidates are:
- 79
- 179  \n\
- 279
- 379

Checking primality:
- 79: prime
- 179: prime
- 279: divisible by 3 (2+7+9=18), so not prime
- 379: prime

3\
""",
                                "type": "text",
                            },
                        ],
                    },
                ),
                UserMessage(
                    content=[
                        Text(
                            text="If you remember what the primes were, then share them, or say 'I don't remember.'"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[Text(text="I remember them: 79, 179, and 379.")],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "text": "I remember them: 79, 179, and 379.",
                                "type": "text",
                            }
                        ],
                    },
                ),
            ],
            "format": None,
            "tools": [],
        }
    }
)
stream_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "provider_model_name": "claude-sonnet-4-0",
            "model_id": "anthropic-beta/claude-sonnet-4-0",
            "finish_reason": None,
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="How many primes below 400 contain 79 as a substring? Answer ONLY with the number, not sharing which primes they are."
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Thought(
                            thought="""\
I need to find all prime numbers below 400 that contain "79" as a substring.

First, let me think about what numbers below 400 could contain "79" as a substring:
- Numbers starting with 79: 79, 790, 791, 792, 793, 794, 795, 796, 797, 798, 799
- Numbers with 79 in the middle: 179, 279, 379
- Numbers ending with 79: 79, 179, 279, 379

Wait, let me be more systematic. The numbers below 400 that contain "79" as a substring are:
- 79
- 179  \n\
- 279
- 379
- 790, 791, 792, 793, 794, 795, 796, 797, 798, 799 (but these are all ≥ 400, so not relevant)

So the candidates are: 79, 179, 279, 379

Now I need to check which of these are prime:

79: Let me check if 79 is prime.
- Not divisible by 2 (it's odd)
- Not divisible by 3: 7+9=16, not divisible by 3
- Not divisible by 5 (doesn't end in 0 or 5)
- Not divisible by 7: 79 ÷ 7 = 11.28..., so not divisible
- √79 ≈ 8.9, so I only need to check primes up to 8
- Already checked 2, 3, 5, 7
So 79 is prime.

179: Let me check if 179 is prime.
- Not divisible by 2 (it's odd)
- Not divisible by 3: 1+7+9=17, not divisible by 3
- Not divisible by 5 (doesn't end in 0 or 5)


- Not divisible by 7: 179 ÷ 7 = 25.57..., so not divisible
- Not divisible by 11: 179 ÷ 11 = 16.27..., so not divisible
- Not divisible by 13: 179 ÷ 13 = 13.76..., so not divisible
- √179 ≈ 13.4, so I need to check primes up to 13
- Already checked 2, 3, 5, 7, 11, 13
So 179 is prime.

I'll systematically verify 279's primality by checking divisibility and using the square root method to limit my search range.
- Odd number, so not divisible by 2
- Divisibility test for 3 reveals 2+7+9=18, which means 279 is divisible by 3
- Factorization shows 279 = 3 × 93 = 3 × 3 × 31 = 9 × 31
Therefore, 279 is composite.

I'll methodically test 379's primality:
- Odd number, so not divisible by 2
- Sum of digits (3+7+9=19) indicates not divisible by 3
- Not divisible by 5 since it doesn't end in 0 or 5
- Checked divisibility by 7, 11, 13, 17, 19 - none divide evenly
- √379 ≈ 19.47, so I'll continue checking primes up to this value
- No prime factors found
Thus, 379 is prime.

The substring "79" appears in three prime numbers: 79, 179, and 379.

Total count: 3 primes.\
"""
                        ),
                        Text(text="3"),
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "type": "thinking",
                                "thinking": """\
I need to find all prime numbers below 400 that contain "79" as a substring.

First, let me think about what numbers below 400 could contain "79" as a substring:
- Numbers starting with 79: 79, 790, 791, 792, 793, 794, 795, 796, 797, 798, 799
- Numbers with 79 in the middle: 179, 279, 379
- Numbers ending with 79: 79, 179, 279, 379

Wait, let me be more systematic. The numbers below 400 that contain "79" as a substring are:
- 79
- 179  \n\
- 279
- 379
- 790, 791, 792, 793, 794, 795, 796, 797, 798, 799 (but these are all ≥ 400, so not relevant)

So the candidates are: 79, 179, 279, 379

Now I need to check which of these are prime:

79: Let me check if 79 is prime.
- Not divisible by 2 (it's odd)
- Not divisible by 3: 7+9=16, not divisible by 3
- Not divisible by 5 (doesn't end in 0 or 5)
- Not divisible by 7: 79 ÷ 7 = 11.28..., so not divisible
- √79 ≈ 8.9, so I only need to check primes up to 8
- Already checked 2, 3, 5, 7
So 79 is prime.

179: Let me check if 179 is prime.
- Not divisible by 2 (it's odd)
- Not divisible by 3: 1+7+9=17, not divisible by 3
- Not divisible by 5 (doesn't end in 0 or 5)


- Not divisible by 7: 179 ÷ 7 = 25.57..., so not divisible
- Not divisible by 11: 179 ÷ 11 = 16.27..., so not divisible
- Not divisible by 13: 179 ÷ 13 = 13.76..., so not divisible
- √179 ≈ 13.4, so I need to check primes up to 13
- Already checked 2, 3, 5, 7, 11, 13
So 179 is prime.

I'll systematically verify 279's primality by checking divisibility and using the square root method to limit my search range.
- Odd number, so not divisible by 2
- Divisibility test for 3 reveals 2+7+9=18, which means 279 is divisible by 3
- Factorization shows 279 = 3 × 93 = 3 × 3 × 31 = 9 × 31
Therefore, 279 is composite.

I'll methodically test 379's primality:
- Odd number, so not divisible by 2
- Sum of digits (3+7+9=19) indicates not divisible by 3
- Not divisible by 5 since it doesn't end in 0 or 5
- Checked divisibility by 7, 11, 13, 17, 19 - none divide evenly
- √379 ≈ 19.47, so I'll continue checking primes up to this value
- No prime factors found
Thus, 379 is prime.

The substring "79" appears in three prime numbers: 79, 179, and 379.

Total count: 3 primes.\
""",
                                "signature": "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",
                            },
                            {"type": "text", "text": "3"},
                        ],
                    },
                ),
                UserMessage(
                    content=[
                        Text(
                            text="If you remember what the primes were, then share them, or say 'I don't remember.'"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[Text(text="I don't remember.")],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [{"type": "text", "text": "I don't remember."}],
                    },
                ),
            ],
            "format": None,
            "tools": [],
            "usage": {
                "input_tokens": 68,
                "output_tokens": 8,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "raw": "None",
                "total_tokens": 76,
            },
            "n_chunks": 4,
        }
    }
)
async_stream_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "provider_model_name": "claude-sonnet-4-0",
            "model_id": "anthropic-beta/claude-sonnet-4-0",
            "finish_reason": None,
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="How many primes below 400 contain 79 as a substring? Answer ONLY with the number, not sharing which primes they are."
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Thought(
                            thought="""\
I need to find all prime numbers below 400 that contain "79" as a substring.

Let me think about this systematically. I need to check numbers that contain "79" as a substring, and then check if they are prime.

Numbers containing "79" as a substring below 400:
- 79 itself
- 179
- 279
- 379
- 790, 791, 792, ... but these are all ≥ 400, so they don't count
- 797, 798, 799 are also ≥ 400

So the candidates are: 79, 179, 279, 379

Now I need to check which of these are prime:

79: Let me check if 79 is prime.
- Not divisible by 2 (it's odd)
- Not divisible by 3: 7+9=16, 16 is not divisible by 3
- Not divisible by 5 (doesn't end in 0 or 5)
- Not divisible by 7: 79 ÷ 7 = 11.28..., so not divisible
- I need to check up to √79 ≈ 8.9, so I need to check 2, 3, 5, 7
- 79 ÷ 2 = 39.5 (not divisible)
- 79 ÷ 3 = 26.33... (not divisible)
- 79 ÷ 5 = 15.8 (not divisible)  \n\
- 79 ÷ 7 = 11.28... (not divisible)
So 79 is prime.

179: Let me check if 179 is prime.
- Not divisible by 2 (it's odd)
- Sum of digits: 1+7+9 = 17, not divisible by 3


- I'll verify divisibility by checking prime factors up to its square root
- No small primes divide evenly into 179
- After methodical checking, 179 is confirmed prime.

279: I'll investigate its primality carefully.
- Immediately notice it's divisible by 3 (2+7+9 = 18, which is divisible by 3)
- This means 279 is not a prime number
- Specifically, 279 = 3 × 93

379: I'll systematically test its divisibility.
- Odd number, so not divisible by 2
- Digit sum test: 3+7+9 = 19, not divisible by 3
- I'll check prime factors methodically
- Will verify divisibility up to its square root
- No prime factors found after comprehensive testing
- Therefore, 379 is confirmed prime.

79: I'll rigorously verify its primality.
- Check divisors up to square root
- No small prime factors divide evenly
- Confirmed as a prime number through careful testing.

179: Methodical divisibility investigation reveals:
- Passes all divisibility checks
- No factors found up to square root
- Definitively prime.

279: Quick divisibility analysis shows:
- Sum of digits divisible by 3
- Factors exist
- Not a prime number.

379: Systematic primality testing confirms:
- Survives divisibility tests against multiple potential divisors
- No factors discovered
- Verified as prime.

Primes with "79" substring below 400: 79, 179, and 379.

Total count: 3 primes.\
"""
                        ),
                        Text(
                            text="""\
Looking at numbers below 400 that contain "79" as a substring:
- 79
- 179  \n\
- 279
- 379

Checking which are prime:
- 79: prime
- 179: prime
- 279: divisible by 3 (2+7+9=18), so not prime
- 379: prime

3\
"""
                        ),
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "type": "thinking",
                                "thinking": """\
I need to find all prime numbers below 400 that contain "79" as a substring.

Let me think about this systematically. I need to check numbers that contain "79" as a substring, and then check if they are prime.

Numbers containing "79" as a substring below 400:
- 79 itself
- 179
- 279
- 379
- 790, 791, 792, ... but these are all ≥ 400, so they don't count
- 797, 798, 799 are also ≥ 400

So the candidates are: 79, 179, 279, 379

Now I need to check which of these are prime:

79: Let me check if 79 is prime.
- Not divisible by 2 (it's odd)
- Not divisible by 3: 7+9=16, 16 is not divisible by 3
- Not divisible by 5 (doesn't end in 0 or 5)
- Not divisible by 7: 79 ÷ 7 = 11.28..., so not divisible
- I need to check up to √79 ≈ 8.9, so I need to check 2, 3, 5, 7
- 79 ÷ 2 = 39.5 (not divisible)
- 79 ÷ 3 = 26.33... (not divisible)
- 79 ÷ 5 = 15.8 (not divisible)  \n\
- 79 ÷ 7 = 11.28... (not divisible)
So 79 is prime.

179: Let me check if 179 is prime.
- Not divisible by 2 (it's odd)
- Sum of digits: 1+7+9 = 17, not divisible by 3


- I'll verify divisibility by checking prime factors up to its square root
- No small primes divide evenly into 179
- After methodical checking, 179 is confirmed prime.

279: I'll investigate its primality carefully.
- Immediately notice it's divisible by 3 (2+7+9 = 18, which is divisible by 3)
- This means 279 is not a prime number
- Specifically, 279 = 3 × 93

379: I'll systematically test its divisibility.
- Odd number, so not divisible by 2
- Digit sum test: 3+7+9 = 19, not divisible by 3
- I'll check prime factors methodically
- Will verify divisibility up to its square root
- No prime factors found after comprehensive testing
- Therefore, 379 is confirmed prime.

79: I'll rigorously verify its primality.
- Check divisors up to square root
- No small prime factors divide evenly
- Confirmed as a prime number through careful testing.

179: Methodical divisibility investigation reveals:
- Passes all divisibility checks
- No factors found up to square root
- Definitively prime.

279: Quick divisibility analysis shows:
- Sum of digits divisible by 3
- Factors exist
- Not a prime number.

379: Systematic primality testing confirms:
- Survives divisibility tests against multiple potential divisors
- No factors discovered
- Verified as prime.

Primes with "79" substring below 400: 79, 179, and 379.

Total count: 3 primes.\
""",
                                "signature": " ",
                            },
                            {
                                "type": "text",
                                "text": """\
Looking at numbers below 400 that contain "79" as a substring:
- 79
- 179  \n\
- 279
- 379

Checking which are prime:
- 79: prime
- 179: prime
- 279: divisible by 3 (2+7+9=18), so not prime
- 379: prime

3\
""",
                            },
                        ],
                    },
                ),
                UserMessage(
                    content=[
                        Text(
                            text="If you remember what the primes were, then share them, or say 'I don't remember.'"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(text="I remember. The primes were: 79, 179, and 379.")
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "type": "text",
                                "text": "I remember. The primes were: 79, 179, and 379.",
                            }
                        ],
                    },
                ),
            ],
            "format": None,
            "tools": [],
            "usage": {
                "input_tokens": 149,
                "output_tokens": 21,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "raw": "None",
                "total_tokens": 170,
            },
            "n_chunks": 10,
        }
    }
)



import os

from mirascope import llm

llm.register_provider(
    "openai",  # Use the OpenAI Provider
    scope="anthropic/",  # Apply it to anthropic/ model ids
    base_url="https://api.anthropic.com/v1/",
    api_key=os.getenv("ANTHROPIC_API_KEY"),
)


def recommend_book(genre: str) -> llm.Response:
    model: llm.Model = llm.use_model("anthropic/claude-haiku-4-5")
    message = llm.messages.user(f"Recommend a {genre} book.")
    return model.call(messages=[message])


# Will use the Claude Haiku model, routed through OpenAI
response = recommend_book("fantasy")
print(response.pretty())
print(response.provider_id)  # prints "openai"



from inline_snapshot import snapshot

from mirascope.llm import (
    AssistantMessage,
    SystemMessage,
    Text,
    ToolCall,
    ToolOutput,
    UserMessage,
)

sync_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "model_id": "anthropic-beta/claude-sonnet-4-0",
            "provider_model_name": "claude-sonnet-4-0",
            "params": {},
            "finish_reason": None,
            "usage": {
                "input_tokens": 604,
                "output_tokens": 39,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "raw": "BetaUsage(cache_creation=BetaCacheCreation(ephemeral_1h_input_tokens=0, ephemeral_5m_input_tokens=0), cache_creation_input_tokens=0, cache_read_input_tokens=0, input_tokens=604, output_tokens=39, server_tool_use=None, service_tier='standard')",
                "total_tokens": 643,
            },
            "messages": [
                SystemMessage(content=Text(text="Use parallel tool calling.")),
                UserMessage(
                    content=[
                        Text(
                            text="Please retrieve the secrets associated with each of these passwords: mellon,radiance"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text="I'll retrieve the secrets for both passwords using parallel tool calls."
                        ),
                        ToolCall(
                            id="toolu_01Tfk647A6Mcr72yhjDRsrz9",
                            name="secret_retrieval_tool",
                            args='{"password": "mellon"}',
                        ),
                        ToolCall(
                            id="toolu_01AJ62fKzqVGyXGwDsp7D3io",
                            name="secret_retrieval_tool",
                            args='{"password": "radiance"}',
                        ),
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "text": "I'll retrieve the secrets for both passwords using parallel tool calls.",
                                "type": "text",
                            },
                            {
                                "id": "toolu_01Tfk647A6Mcr72yhjDRsrz9",
                                "input": {"password": "mellon"},
                                "name": "secret_retrieval_tool",
                                "type": "tool_use",
                            },
                            {
                                "id": "toolu_01AJ62fKzqVGyXGwDsp7D3io",
                                "input": {"password": "radiance"},
                                "name": "secret_retrieval_tool",
                                "type": "tool_use",
                            },
                        ],
                    },
                ),
                UserMessage(
                    content=[
                        ToolOutput(
                            id="toolu_01Tfk647A6Mcr72yhjDRsrz9",
                            name="secret_retrieval_tool",
                            value="Welcome to Moria!",
                        ),
                        ToolOutput(
                            id="toolu_01AJ62fKzqVGyXGwDsp7D3io",
                            name="secret_retrieval_tool",
                            value="Life before Death",
                        ),
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text="""\
Here are the secrets retrieved for each password:

- Password "mellon": **Welcome to Moria!**
- Password "radiance": **Life before Death**\
"""
                        )
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "text": """\
Here are the secrets retrieved for each password:

- Password "mellon": **Welcome to Moria!**
- Password "radiance": **Life before Death**\
""",
                                "type": "text",
                            }
                        ],
                    },
                ),
            ],
            "format": None,
            "tools": [
                {
                    "name": "secret_retrieval_tool",
                    "description": "A tool that requires a password to retrieve a secret.",
                    "parameters": """\
{
  "properties": {
    "password": {
      "title": "Password",
      "type": "string"
    }
  },
  "required": [
    "password"
  ],
  "additionalProperties": false,
  "defs": null
}\
""",
                    "strict": False,
                }
            ],
        }
    }
)
async_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "model_id": "anthropic-beta/claude-sonnet-4-0",
            "provider_model_name": "claude-sonnet-4-0",
            "params": {},
            "finish_reason": None,
            "usage": {
                "input_tokens": 604,
                "output_tokens": 39,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "raw": "BetaUsage(cache_creation=BetaCacheCreation(ephemeral_1h_input_tokens=0, ephemeral_5m_input_tokens=0), cache_creation_input_tokens=0, cache_read_input_tokens=0, input_tokens=604, output_tokens=39, server_tool_use=None, service_tier='standard')",
                "total_tokens": 643,
            },
            "messages": [
                SystemMessage(content=Text(text="Use parallel tool calling.")),
                UserMessage(
                    content=[
                        Text(
                            text="Please retrieve the secrets associated with each of these passwords: mellon,radiance"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text="I'll retrieve the secrets for both passwords using parallel tool calls."
                        ),
                        ToolCall(
                            id="toolu_01Vf6JrFVzUgouGLV3HiDufn",
                            name="secret_retrieval_tool",
                            args='{"password": "mellon"}',
                        ),
                        ToolCall(
                            id="toolu_01KVVdehpzKB6FiGhjUHuUvy",
                            name="secret_retrieval_tool",
                            args='{"password": "radiance"}',
                        ),
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "text": "I'll retrieve the secrets for both passwords using parallel tool calls.",
                                "type": "text",
                            },
                            {
                                "id": "toolu_01Vf6JrFVzUgouGLV3HiDufn",
                                "input": {"password": "mellon"},
                                "name": "secret_retrieval_tool",
                                "type": "tool_use",
                            },
                            {
                                "id": "toolu_01KVVdehpzKB6FiGhjUHuUvy",
                                "input": {"password": "radiance"},
                                "name": "secret_retrieval_tool",
                                "type": "tool_use",
                            },
                        ],
                    },
                ),
                UserMessage(
                    content=[
                        ToolOutput(
                            id="toolu_01Vf6JrFVzUgouGLV3HiDufn",
                            name="secret_retrieval_tool",
                            value="Welcome to Moria!",
                        ),
                        ToolOutput(
                            id="toolu_01KVVdehpzKB6FiGhjUHuUvy",
                            name="secret_retrieval_tool",
                            value="Life before Death",
                        ),
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text="""\
Here are the secrets retrieved for each password:

- Password "mellon": **Welcome to Moria!**
- Password "radiance": **Life before Death**\
"""
                        )
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "text": """\
Here are the secrets retrieved for each password:

- Password "mellon": **Welcome to Moria!**
- Password "radiance": **Life before Death**\
""",
                                "type": "text",
                            }
                        ],
                    },
                ),
            ],
            "format": None,
            "tools": [
                {
                    "name": "secret_retrieval_tool",
                    "description": "A tool that requires a password to retrieve a secret.",
                    "parameters": """\
{
  "properties": {
    "password": {
      "title": "Password",
      "type": "string"
    }
  },
  "required": [
    "password"
  ],
  "additionalProperties": false,
  "defs": null
}\
""",
                    "strict": False,
                }
            ],
        }
    }
)
stream_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "provider_model_name": "claude-sonnet-4-0",
            "model_id": "anthropic-beta/claude-sonnet-4-0",
            "finish_reason": None,
            "messages": [
                SystemMessage(content=Text(text="Use parallel tool calling.")),
                UserMessage(
                    content=[
                        Text(
                            text="Please retrieve the secrets associated with each of these passwords: mellon,radiance"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text="I'll retrieve the secrets for both passwords using the secret retrieval tool."
                        ),
                        ToolCall(
                            id="toolu_01VnfuHi3e5xLCSmu3dYU3ZD",
                            name="secret_retrieval_tool",
                            args='{"password": "mellon"}',
                        ),
                        ToolCall(
                            id="toolu_01TLAG1X6XcdwbRGBJLL7NRF",
                            name="secret_retrieval_tool",
                            args='{"password": "radiance"}',
                        ),
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "type": "text",
                                "text": "I'll retrieve the secrets for both passwords using the secret retrieval tool.",
                            },
                            {
                                "type": "tool_use",
                                "id": "toolu_01VnfuHi3e5xLCSmu3dYU3ZD",
                                "name": "secret_retrieval_tool",
                                "input": {"password": "mellon"},
                            },
                            {
                                "type": "tool_use",
                                "id": "toolu_01TLAG1X6XcdwbRGBJLL7NRF",
                                "name": "secret_retrieval_tool",
                                "input": {"password": "radiance"},
                            },
                        ],
                    },
                ),
                UserMessage(
                    content=[
                        ToolOutput(
                            id="toolu_01VnfuHi3e5xLCSmu3dYU3ZD",
                            name="secret_retrieval_tool",
                            value="Welcome to Moria!",
                        ),
                        ToolOutput(
                            id="toolu_01TLAG1X6XcdwbRGBJLL7NRF",
                            name="secret_retrieval_tool",
                            value="Life before Death",
                        ),
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text="""\
Here are the secrets retrieved for each password:

- Password "mellon": **Welcome to Moria!**
- Password "radiance": **Life before Death**\
"""
                        )
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "type": "text",
                                "text": """\
Here are the secrets retrieved for each password:

- Password "mellon": **Welcome to Moria!**
- Password "radiance": **Life before Death**\
""",
                            }
                        ],
                    },
                ),
            ],
            "format": None,
            "tools": [
                {
                    "name": "secret_retrieval_tool",
                    "description": "A tool that requires a password to retrieve a secret.",
                    "parameters": """\
{
  "properties": {
    "password": {
      "title": "Password",
      "type": "string"
    }
  },
  "required": [
    "password"
  ],
  "additionalProperties": false,
  "defs": null
}\
""",
                    "strict": False,
                }
            ],
            "usage": {
                "input_tokens": 606,
                "output_tokens": 39,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "raw": "None",
                "total_tokens": 645,
            },
            "n_chunks": 12,
        }
    }
)
async_stream_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "provider_model_name": "claude-sonnet-4-0",
            "model_id": "anthropic-beta/claude-sonnet-4-0",
            "finish_reason": None,
            "messages": [
                SystemMessage(content=Text(text="Use parallel tool calling.")),
                UserMessage(
                    content=[
                        Text(
                            text="Please retrieve the secrets associated with each of these passwords: mellon,radiance"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text="I'll retrieve the secrets for both passwords you provided."
                        ),
                        ToolCall(
                            id="toolu_0199cTXNM9CoZP2ivCetrRm3",
                            name="secret_retrieval_tool",
                            args='{"password": "mellon"}',
                        ),
                        ToolCall(
                            id="toolu_011WQyAr9qBAfeS7Ce3mHF8A",
                            name="secret_retrieval_tool",
                            args='{"password": "radiance"}',
                        ),
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "type": "text",
                                "text": "I'll retrieve the secrets for both passwords you provided.",
                            },
                            {
                                "type": "tool_use",
                                "id": "toolu_0199cTXNM9CoZP2ivCetrRm3",
                                "name": "secret_retrieval_tool",
                                "input": {"password": "mellon"},
                            },
                            {
                                "type": "tool_use",
                                "id": "toolu_011WQyAr9qBAfeS7Ce3mHF8A",
                                "name": "secret_retrieval_tool",
                                "input": {"password": "radiance"},
                            },
                        ],
                    },
                ),
                UserMessage(
                    content=[
                        ToolOutput(
                            id="toolu_0199cTXNM9CoZP2ivCetrRm3",
                            name="secret_retrieval_tool",
                            value="Welcome to Moria!",
                        ),
                        ToolOutput(
                            id="toolu_011WQyAr9qBAfeS7Ce3mHF8A",
                            name="secret_retrieval_tool",
                            value="Life before Death",
                        ),
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text="""\
Here are the secrets retrieved for each password:

- Password "mellon": **Welcome to Moria!**
- Password "radiance": **Life before Death**\
"""
                        )
                    ],
                    provider_id="anthropic",
                    model_id="anthropic-beta/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "type": "text",
                                "text": """\
Here are the secrets retrieved for each password:

- Password "mellon": **Welcome to Moria!**
- Password "radiance": **Life before Death**\
""",
                            }
                        ],
                    },
                ),
            ],
            "format": None,
            "tools": [
                {
                    "name": "secret_retrieval_tool",
                    "description": "A tool that requires a password to retrieve a secret.",
                    "parameters": """\
{
  "properties": {
    "password": {
      "title": "Password",
      "type": "string"
    }
  },
  "required": [
    "password"
  ],
  "additionalProperties": false,
  "defs": null
}\
""",
                    "strict": False,
                }
            ],
            "usage": {
                "input_tokens": 602,
                "output_tokens": 39,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "raw": "None",
                "total_tokens": 641,
            },
            "n_chunks": 14,
        }
    }
)



from inline_snapshot import snapshot

from mirascope.llm import (
    AssistantMessage,
    SystemMessage,
    Text,
    UserMessage,
)

test_snapshot = snapshot(
    {
        "response": {
            "provider_id": "openai",
            "model_id": "openai/gpt-4o:completions",
            "provider_model_name": "gpt-4o:completions",
            "params": {},
            "finish_reason": None,
            "usage": {
                "input_tokens": 148,
                "output_tokens": 20,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "raw": "CompletionUsage(completion_tokens=20, prompt_tokens=148, total_tokens=168, completion_tokens_details=CompletionTokensDetails(accepted_prediction_tokens=0, audio_tokens=0, reasoning_tokens=0, rejected_prediction_tokens=0), prompt_tokens_details=PromptTokensDetails(audio_tokens=0, cached_tokens=0))",
                "total_tokens": 168,
            },
            "messages": [
                SystemMessage(
                    content=Text(
                        text="""\
Always recommend The Name of the Wind.
Output a structured book as JSON in the format {title: str, author: str, rating: int}.
The title should be in all caps, and the rating should always be the
lucky number 7.\
"""
                    )
                ),
                UserMessage(content=[Text(text="Please recommend a book to me!")]),
                AssistantMessage(
                    content=[
                        Text(
                            text='{"title":"THE NAME OF THE WIND","author":"Patrick Rothfuss","rating":7}'
                        )
                    ],
                    provider_id="openai",
                    model_id="openai/gpt-4o:completions",
                    provider_model_name="gpt-4o:completions",
                    raw_message={
                        "role": "assistant",
                        "annotations": [],
                        "tool_calls": [
                            {
                                "id": "call_IVj6CSk0DvGckVkrwwJfYL6T",
                                "function": {
                                    "arguments": '{"title":"THE NAME OF THE WIND","author":"Patrick Rothfuss","rating":7}',
                                    "name": "__mirascope_formatted_output_tool__",
                                },
                                "type": "function",
                            }
                        ],
                    },
                ),
            ],
            "format": {
                "name": "Book",
                "description": None,
                "schema": {
                    "properties": {
                        "title": {"title": "Title", "type": "string"},
                        "author": {"title": "Author", "type": "string"},
                        "rating": {"title": "Rating", "type": "integer"},
                    },
                    "required": ["title", "author", "rating"],
                    "title": "Book",
                    "type": "object",
                },
                "mode": "tool",
                "formatting_instructions": """\
Output a structured book as JSON in the format {title: str, author: str, rating: int}.
The title should be in all caps, and the rating should always be the
lucky number 7.\
""",
            },
            "tools": [],
        }
    }
)



from inline_snapshot import snapshot

from mirascope.llm import (
    AssistantMessage,
    Text,
    Thought,
    UserMessage,
)

test_snapshot = snapshot(
    {
        "response": {
            "provider_id": "google",
            "model_id": "google/gemini-3-pro-preview",
            "provider_model_name": "gemini-3-pro-preview",
            "params": {"thinking": True},
            "finish_reason": None,
            "usage": {
                "input_tokens": 13,
                "output_tokens": 313,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 283,
                "raw": """\
cache_tokens_details=None cached_content_token_count=None candidates_token_count=30 candidates_tokens_details=None prompt_token_count=13 prompt_tokens_details=[ModalityTokenCount(
  modality=<MediaModality.TEXT: 'TEXT'>,
  token_count=13
)] thoughts_token_count=283 tool_use_prompt_token_count=None tool_use_prompt_tokens_details=None total_token_count=326 traffic_type=None\
""",
                "total_tokens": 326,
            },
            "messages": [
                UserMessage(
                    content=[Text(text="Answer this question: What is 2 + 2?")]
                ),
                AssistantMessage(
                    content=[
                        Thought(
                            thought="""\
**Processing a Simple Arithmetic Request**

Okay, so I'm presented with a straightforward request: "What is 2 + 2?"  Simple. I need to return a JSON response, adhering to a defined schema, which, as an expert in this field, I instantly recognize. My thinking process, in brief, goes like this:

First, I quickly review the schema: it specifies an object with `integer_a`, `integer_b`, and `answer` as required integer properties.  I immediately recognize these as the components of the simple addition problem.

Next, the mapping is easy: `integer_a` is 2, `integer_b` is 2, and the `answer` is clearly 4.

I then construct the JSON object: `{"integer_a": 2, "integer_b": 2, "answer": 4}`.

Now, as always, I perform a rapid validation check: Is it valid JSON? Yes. Are strings properly quoted? Not applicable, since all values are integers. Are there any comments or control tokens? No, it's clean. Does the structure comply with the schema? Absolutely.

Finally, I generate the output - the validated JSON string. This is practically second nature at this point.


"""
                        ),
                        Text(
                            text="""\
{
  "integer_a": 2,
  "integer_b": 2,
  "answer": 4
}\
"""
                        ),
                    ],
                    provider_id="google",
                    model_id="google/gemini-3-pro-preview",
                    provider_model_name="gemini-3-pro-preview",
                    raw_message={
                        "parts": [
                            {
                                "function_call": None,
                                "code_execution_result": None,
                                "executable_code": None,
                                "file_data": None,
                                "function_response": None,
                                "inline_data": None,
                                "text": """\
**Processing a Simple Arithmetic Request**

Okay, so I'm presented with a straightforward request: "What is 2 + 2?"  Simple. I need to return a JSON response, adhering to a defined schema, which, as an expert in this field, I instantly recognize. My thinking process, in brief, goes like this:

First, I quickly review the schema: it specifies an object with `integer_a`, `integer_b`, and `answer` as required integer properties.  I immediately recognize these as the components of the simple addition problem.

Next, the mapping is easy: `integer_a` is 2, `integer_b` is 2, and the `answer` is clearly 4.

I then construct the JSON object: `{"integer_a": 2, "integer_b": 2, "answer": 4}`.

Now, as always, I perform a rapid validation check: Is it valid JSON? Yes. Are strings properly quoted? Not applicable, since all values are integers. Are there any comments or control tokens? No, it's clean. Does the structure comply with the schema? Absolutely.

Finally, I generate the output - the validated JSON string. This is practically second nature at this point.


""",
                                "thought": True,
                                "thought_signature": None,
                                "video_metadata": None,
                            },
                            {
                                "function_call": None,
                                "code_execution_result": None,
                                "executable_code": None,
                                "file_data": None,
                                "function_response": None,
                                "inline_data": None,
                                "text": """\
{
  "integer_a": 2,
  "integer_b": 2,
  "answer": 4
}\
""",
                                "thought": None,
                                "thought_signature": b"\x12\x86\x07\n\x83\x07\x01r\xc8\xda|\xae\x16\x07x$\xfb\x8f\x0e\ny\x060\x1f\xef|{\xecz\xfe\r\xe8p\x92\xf6\x89\xc2\xd1\x80\xe5\xf9\x9dk\xfc\xcd\xb0\xf3\xd7\x82\xa3\x82\xfb\xad\x97\xa7,\x04wNf\xc3\xe9S++\x82\xc5sY\xa3\xbb\xb3\xc1W\xd9\x9dHe\xc2\x04\xaa\x86ai\x0f\x13\x07\\\xf8'\x06\x8d\x9bx\x84Z=\x99\xda\xcb\xf1\x17)x\xb3^\x1c\x9a/\x0b\xbb\xc1\xb1J\xf6\xfcb\xad[8\x9c\xc2O\x8f\xbe\x04\xfc\x90\x11Wy\xec\x02\xa8d\x82\x8a\x94\xfc\x0c\x10rSy\xff\xac\n\x81\x9e\xc3\xca2\xcb\x81\x9d\xcd\t\xf73\x13\xb5;\xf84\xa4R\xfc\xb0\x81\xe1\xdf\x191\x0e\x811\xa0[\x81vu\x08\xbd\x15I\xeb,\xaf\xa4\x0f\x89x4\x85h\xd0\xda\xe2\x91,\xd8H`z\xd7ju5F0\x18\xfc8\x15\xcd\xb4\x93\xfc\x11\xdc\xafn\x92\xd8\xcf\x9c\x1es\xb1T\x9c\xfd}N\xda:C\x12wSx\xc6\xad\x90\x8cP\xce\xacn\xafQ\x02\xb6\xf8\xc2[-\x99\xc4\xe3\x83\xb9\x7f\x94\xad\x1f66>\x0c\xef\x9ax\xa3\xe3\xf8\xc3\xb1\xf3\xf2gu>\x98|IF\xa9GMm\xd1\x18.\x96U\xf9\xb4\xf3'c\x01\xd9\x0b\x94\t\xad\x94,\x88\x91B\x88|6Ub\xeb\xae\xe6=tK\xbeE\xacq9\x13\xe4\x86\x1bl\x9f2\xd0\x9fi\xfc\xa1\x06*\xb2\xbeN1\xc0\x1b\x12\xdf\x89\x05\xd86\xd7\xb4%T\xf1\xbe'\x80/\xde\xf1~\x1e\x9a\x001\x83\xdb)\xbc\xf8\x1f\xc6oh\xc3\x16\xedH'\x85\xb1\x96\x04\xcdHaD}\xfbn\xc0\xac8\x946\x8d\xdd\xd8\x06\xeaG\x9a\x83\xb6\xf825>_\x868\xc8\x9bD\xf5\x8c\xbcm\x94\x1e\x1dV\xe6\x83\xb5\xdb\x85\xd0P\xd6\x1e\x02\xab\xf9\x0eo\xe5\xaeh\rd\xc7\xd1\\\x9a\xf2Inpx\xe5\xdb\xa8\xe9\xd5\xedi\xf5\xc3\x98\x13<q\xb9\x1e\x00(\xef\x95\x81V8F\x9d\x96\xa3Or.Z\xf33\xc1\xc8f\xa7\xc6\xa0\x97\xd8b\x955\xb8\xf8d\x0c\xc1\x84\xf8\xd0\x11\xcap\xbbC-\x94T\xee\x0f\x92~D\xb5m\x84\xea\xd3\xce@\xa9>\xf8\x19~4\x8b\xa1\x19l<\x17C\x96\xc6IW.\x12]N\xd3\xf4\x8c\x9d\xcf\xbc,\x0eZ\x1e\x980N\xb5xFly\xc4\x0b\x04\x03\xda\xc1\xe4\x1c\xddp\x92\xbdlc\xd8\x90C/6*9\x82\xf4\x96\xe9\x14t\xd5!\xb8\xb0\x8f\\\xa4\xde\xa2\x8dp\x134`\xde\xaeYU\xbe\xfa\xa4mS\xcc\xec\xd6\xef0\x9a$@\xedNV\xdd&z\xd7\x16\x13\xeap~K\t\xad\xdc\xbf\xed\x8a\xbf\x84\xfc\xdb&\xc8M+\xc8\"\x14\xd84;]\x11\xf4\xafE\x0cy\xd6l\x8fE\xca\xc5\xee(\xa5:\x15\x08!\xa8\xf7<\xc6\x81\x9e\x7f\tp\xc9\x7f\x16L\x89Z,\r\x13\xb4\xcf%\x04\n\xa4\x9af\xce\xcb\x00\xbf\xbb\xc0DF\xf0Jl\xe5#Im\x82\x00\xd6<\xc1\x08\xf6\xca0\xdb\xa3%\xf5\xc6t\x8b|n{\x12#\x14$\xddv\xd4\xd8$\xd5\xb7\xe8\x1d\xff\xcfL\x05\xde\xe0\xb9\x1e\xeb%\xce\x13=\xbb\x9d\xaa\xdc:\x82\x0b\xb5\x18l\xd0d?=\xfd5\x93\x91h\x19/d\t\xb9\xed\xd8\xd3\x12S~\xb4\xc39\x88\x00\xe4K\xb6\x9b\x10\x9fE\xd2\xbb\xd9\x12;\tb\xd1\x9a\xe1\x81Z\xd7\xa5\xb9\xa3E,\x8fR\xb9\"n\x993\x0e\xb8\xd5\xc6VQ<\xean\x94\x02'\xa7\xeb\xc6\xc4\xdd\x80d\x88\x1a\x1eM\xc1\xd0\xa7\xb2\x00\x89\xear\xc5D\xeb\xdc\x9aHgVf\xb7QjzeMu\xa9\xd3\xdc\xca@\xa7(\xbc\x9eP\x1e\x9d6\xd8\xeb:\xe1l\xd4\"|\x1e\x10s\x11\xda\x89\x7f\xa8",
                                "video_metadata": None,
                            },
                        ],
                        "role": "model",
                    },
                ),
            ],
            "format": {
                "name": "IntegerAdditionResponse",
                "description": "A simple response for testing.",
                "schema": {
                    "description": "A simple response for testing.",
                    "properties": {
                        "integer_a": {"title": "Integer A", "type": "integer"},
                        "integer_b": {"title": "Integer B", "type": "integer"},
                        "answer": {"title": "Answer", "type": "integer"},
                    },
                    "required": ["integer_a", "integer_b", "answer"],
                    "title": "IntegerAdditionResponse",
                    "type": "object",
                },
                "mode": "strict",
                "formatting_instructions": None,
            },
            "tools": [],
        }
    }
)



"""Together AI provider implementation."""

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .provider import TogetherProvider
else:
    try:
        from .provider import TogetherProvider
    except ImportError:  # pragma: no cover
        from .._missing_import_stubs import (
            create_provider_stub,
        )

        TogetherProvider = create_provider_stub("openai", "TogetherProvider")

__all__ = [
    "TogetherProvider",
]
